#!/usr/bin/env python

from qt import *
import qtui
import qtxmldom

class Factory:
    def __init__(self, ui_filename):
        self.ui_filename = ui_filename
        self.ui_doc = qtxmldom.parse(ui_filename)

    def connect(self, widget, obj):

        for connection in self.ui_doc.getElementsByTagName("connection"):
            sender_name = self.get_text(connection.getElementsByTagName("sender")[0]).encode("utf-8")
            signal_name = self.get_text(connection.getElementsByTagName("signal")[0]).encode("utf-8")
            slot_name = self.get_text(connection.getElementsByTagName("slot")[0]).encode("utf-8")

            if widget.name() == sender_name:
                sender = widget
            else:
                sender = self.find_widget(widget, sender_name)

            if sender:
                signal = SIGNAL(signal_name)
                slot = slot_name.split("(")[0]

                if hasattr(obj, slot):
                    QObject.connect(sender, signal, getattr(obj, slot))

    def find_widget(self, widget, name):
        found = widget.child(name)
        if found:
            return found
        else:
            for child in widget.children():
                found = self.find_widget(child, name)
                if found:
                    return found
        return None

    def get_text(self, node):
        node.normalize()
        return node.childNodes[0].nodeValue

    def find_widget_element(self, name):
        for widget in self.ui_doc.getElementsByTagName("widget"):
            for property in widget.getElementsByTagName("property"):
                if property.getAttribute("name") == "name":
                    for cstring in property.getElementsByTagName("cstring"):
                        found_name = self.get_text(cstring)
                        if found_name == name:
                            return widget
        return None

# vim: tabstop=4 expandtab shiftwidth=4
